@@ -10,6 +10,7 @@
 # limitations under the License.
 
 import logging
+import os
 import re
 
 from common.utils import check_command_output, grouper, run_command
@@ -50,8 +51,9 @@ _SQUEUE_FIELDS = [
     "cpus-per-tres",
 ]
 SQUEUE_FIELD_STRING = ",".join([field + ":{size}" for field in _SQUEUE_FIELDS]).format(size=SQUEUE_FIELD_SIZE)
-SCONTROL = "sudo /opt/slurm/bin/scontrol"
-SINFO = "/opt/slurm/bin/sinfo"
+SLURM_BINARIES_DIR = os.environ.get("SLURM_BINARIES_DIR", "/opt/slurm/bin")
+SCONTROL = f"sudo {SLURM_BINARIES_DIR}/scontrol"
+SINFO = f"{SLURM_BINARIES_DIR}/sinfo"
 
 # Set default timeouts for running different slurm commands.
 # These timeouts might be needed when running on large scale
@@ -138,7 +140,7 @@ def update_all_partitions(state, reset_node_addrs_hostname):
                 log.info(f"Setting partition {part.name} state from {part.state} to {state}")
                 if reset_node_addrs_hostname:
                     log.info(f"Resetting partition nodes {part.nodenames}")
-                    set_nodes_down_and_power_save(part.nodenames, reason="stopping cluster")
+                    set_nodes_power_down(part.nodenames, reason="stopping cluster")
                 partition_to_update.append(part.name)
         succeeded_partitions = update_partitions(partition_to_update, state)
         return succeeded_partitions == partition_to_update
@@ -192,9 +194,10 @@ def set_nodes_drain(nodes, reason):
     update_nodes(nodes, state="drain", reason=reason)
 
 
+@retry(stop_max_attempt_number=3, wait_fixed=1500)
 def set_nodes_power_down(nodes, reason=None):
     """Place slurm node into power_down state and reset nodeaddr/nodehostname."""
-    reset_nodes(nodes=nodes, state="power_down", reason=reason, raise_on_error=True)
+    reset_nodes(nodes=nodes, state="power_down_force", reason=reason, raise_on_error=True)
 
 
 def reset_nodes(nodes, state=None, reason=None, raise_on_error=False):
@@ -222,29 +225,18 @@ def set_nodes_idle(nodes, reason=None, reset_node_addrs_hostname=False):
         update_nodes(nodes=nodes, state="resume", reason=reason, raise_on_error=False)
 
 
-@retry(stop_max_attempt_number=3, wait_fixed=1500)
-def set_nodes_down_and_power_save(node_list, reason):
-    """
-    Set slurm nodes into down -> power_down.
-
-    This is the standard failure recovery procedure to reset a CLOUD node.
-    """
-    set_nodes_down(node_list, reason=reason)
-    set_nodes_power_down(node_list, reason=reason)
-
-
 def get_nodes_info(nodes="", command_timeout=DEFAULT_GET_INFO_COMMAND_TIMEOUT):
     """
     Retrieve SlurmNode list from slurm nodelist notation.
 
     Sample slurm nodelist notation: queue1-dy-c5_xlarge-[1-3],queue2-st-t2_micro-5.
     """
-    # awk is used to replace the \n\n record separator with '---\n'
+    # awk is used to replace the \n\n record separator with '######\n'
     # Note: In case the node does not belong to any partition the Partitions field is missing from Slurm output
     show_node_info_command = (
-        f'{SCONTROL} show nodes {nodes} | awk \'BEGIN{{RS="\\n\\n" ; ORS="---\\n";}} {{print}}\' | '
-        'grep -oP "^NodeName=\\K(\\S+)| NodeAddr=\\K(\\S+)| NodeHostName=\\K(\\S+)| State=\\K(\\S+)|'
-        ' Partitions=\\K(\\S+)|(---)"'
+        f'{SCONTROL} show nodes {nodes} | awk \'BEGIN{{RS="\\n\\n" ; ORS="######\\n";}} {{print}}\' | '
+        'grep -oP "^(NodeName=\\S+)|(NodeAddr=\\S+)|(NodeHostName=\\S+)|(State=\\S+)|'
+        '(Partitions=\\S+)|(Reason=.+) |(######)"'
     )
     nodeinfo_str = check_command_output(show_node_info_command, timeout=command_timeout, shell=True)  # nosec
 
@@ -266,6 +258,13 @@ def get_partition_info(command_timeout=DEFAULT_GET_INFO_COMMAND_TIMEOUT, get_all
     ]
 
 
+def resume_powering_down_nodes():
+    """Resume nodes that are powering_down so that are set in power state right away."""
+    log.info("Resuming powering down nodes.")
+    powering_down_nodes = _get_slurm_nodes(states="powering_down")
+    update_nodes(nodes=powering_down_nodes, state="resume", raise_on_error=False)
+
+
 def _parse_partition_name_and_state(partition_info):
     """Parse partition name and state from scontrol output."""
     return grouper(partition_info.splitlines(), 2)
@@ -277,19 +276,21 @@ def _get_all_partition_nodes(partition_name, command_timeout=DEFAULT_GET_INFO_CO
     return check_command_output(show_all_nodes_command, timeout=command_timeout, shell=True).strip()  # nosec
 
 
+def _get_slurm_nodes(states=None, partition_name=None, command_timeout=DEFAULT_GET_INFO_COMMAND_TIMEOUT):
+    sinfo_command = f"{SINFO} -h -N -o %N"
+    if partition_name:
+        sinfo_command += f" -p {partition_name}"
+    if states:
+        sinfo_command += f" -t {states}"
+    # Every node is print on a separate line
+    return check_command_output(sinfo_command, timeout=command_timeout, shell=True).splitlines()  # nosec
+
+
 def _get_partition_nodes(partition_name, command_timeout=DEFAULT_GET_INFO_COMMAND_TIMEOUT):
     """Get up nodes in a parition by querying sinfo, and filtering out power_down nodes."""
-    show_all_nodes_command = f"{SINFO} -h -p {partition_name} -N -o %N"
-    show_power_down_nodes_command = f"{SINFO} -h -p {partition_name} -t power_down,powering_down -N -o %N"
-    show_down_nodes_command = f"{SINFO} -h -p {partition_name} -t down -N -o %N"
-    # Every node is print on a separate line
-    all_nodes = check_command_output(show_all_nodes_command, timeout=command_timeout, shell=True).splitlines()  # nosec
-    power_down_nodes = check_command_output(  # nosec
-        show_power_down_nodes_command, timeout=command_timeout, shell=True
-    ).splitlines()
-    down_nodes = check_command_output(  # nosec
-        show_down_nodes_command, timeout=command_timeout, shell=True
-    ).splitlines()
+    all_nodes = _get_slurm_nodes(partition_name=partition_name)
+    power_down_nodes = _get_slurm_nodes(partition_name=partition_name, states="power_down,powering_down")
+    down_nodes = _get_slurm_nodes(partition_name=partition_name, states="down")
     nodes = []
     for nodename in all_nodes:
         # Always try to maintain the following nodes:
@@ -304,44 +305,60 @@ def _get_partition_nodes(partition_name, command_timeout=DEFAULT_GET_INFO_COMMAN
 def _parse_nodes_info(slurm_node_info):
     """Parse slurm node info into SlurmNode objects."""
     # [ec2-user@ip-10-0-0-58 ~]$ /opt/slurm/bin/scontrol show nodes compute-dy-c5xlarge-[1-3],compute-dy-c5xlarge-50001\
-    # | awk 'BEGIN{RS="\n\n" ; ORS="---\n";} {print}' | grep -oP "^NodeName=\K(\S+)| NodeAddr=\K(\S+)|\
-    # NodeHostName=\K(\S+)| State=\K(\S+)| Partitions=\K(\S+)|(---)"
-    # compute-dy-c5xlarge-1
-    # 1.2.3.4
-    # compute-dy-c5xlarge-1
-    # IDLE+CLOUD+POWER
-    # compute,compute2
-    # ---
-    # compute-dy-c5xlarge-2
-    # 1.2.3.4
-    # compute-dy-c5xlarge-2
-    # IDLE+CLOUD+POWER
-    # compute,compute2
-    # ---
-    # compute-dy-c5xlarge-3
-    # 1.2.3.4
-    # compute-dy-c5xlarge-3
-    # IDLE+CLOUD+POWER
-    # compute,compute2
-    # ---
-    # compute-dy-c5xlarge-50001
-    # 1.2.3.4
-    # compute-dy-c5xlarge-50001
-    # IDLE+CLOUD+POWER
-    # ---
-    node_info = slurm_node_info.split("---")
+    # awk 'BEGIN{{RS="\n\n" ; ORS="######\n";}} {{print}}' | grep -oP "^(NodeName=\S+)|(NodeAddr=\S+)
+    # |(NodeHostName=\S+)|(State=\S+)|(Partitions=\S+)|(Reason=.+) |(######)"
+    # NodeName=compute-dy-c5xlarge-1
+    # NodeAddr=1.2.3.4
+    # NodeHostName=compute-dy-c5xlarge-1
+    # State=IDLE+CLOUD+POWER
+    # Partitions=compute,compute2
+    # Reason=some reason
+    # ######
+    # NodeName=compute-dy-c5xlarge-2
+    # NodeAddr=1.2.3.4
+    # NodeHostName=compute-dy-c5xlarge-2
+    # State=IDLE+CLOUD+POWER
+    # Partitions=compute,compute2
+    # Reason=(Code:InsufficientInstanceCapacity)Failure when resuming nodes
+    # ######
+    # NodeName=compute-dy-c5xlarge-3
+    # NodeAddr=1.2.3.4
+    # NodeHostName=compute-dy-c5xlarge-3
+    # State=IDLE+CLOUD+POWER
+    # Partitions=compute,compute2
+    # ######
+    # NodeName=compute-dy-c5xlarge-50001
+    # NodeAddr=1.2.3.4
+    # NodeHostName=compute-dy-c5xlarge-50001
+    # State=IDLE+CLOUD+POWER
+    # ######
+
+    map_slurm_key_to_arg = {
+        "NodeName": "name",
+        "NodeAddr": "nodeaddr",
+        "NodeHostName": "nodehostname",
+        "State": "state",
+        "Partitions": "partitions",
+        "Reason": "reason",
+    }
+
+    node_info = slurm_node_info.split("######")
     slurm_nodes = []
     for node in node_info:
         lines = node.strip().splitlines()
+        kwargs = {}
+        for line in lines:
+            key, value = line.split("=")
+            kwargs[map_slurm_key_to_arg[key]] = value
         if lines:
             try:
-                if is_static_node(lines[0]):
-                    node = StaticNode(*lines)
+                if is_static_node(kwargs["name"]):
+                    node = StaticNode(**kwargs)
                     slurm_nodes.append(node)
                 else:
-                    node = DynamicNode(*lines)
+                    node = DynamicNode(**kwargs)
                     slurm_nodes.append(node)
             except InvalidNodenameError:
-                log.warning("Ignoring node %s because it has an invalid name", lines[0])
+                log.warning("Ignoring node %s because it has an invalid name", kwargs["name"])
 
     return slurm_nodes
