@@ -39,7 +39,7 @@ def log_exception(
             try:
                 return function(*args, **kwargs)
             except catch_exception as e:
-                logger.log(log_level, "Failed when %s with exception %s", action_desc, e)
+                logger.log(log_level, "Failed when %s with exception %s, message: %s", action_desc, type(e).__name__, e)
                 if raise_on_error:
                     if exception_to_raise:
                         raise exception_to_raise
@@ -59,18 +59,22 @@ def print_with_count(resource_list):
     return f"(x{len(resource_list)}) {str(resource_list)}"
 
 
-def retrieve_instance_type_mapping(file_path):
-    """Retrieve instance type mapping file content."""
+def read_json(file_path, default=None):
+    """Read json file into a dict."""
     try:
         with open(file_path) as mapping_file:
             return json.load(mapping_file)
     except Exception as e:
-        logging.error(
-            "Unable to get instance_type mapping from '%s'. Failed with exception: %s",
-            file_path,
-            e,
-        )
-        raise
+        if default is None:
+            logger.error(
+                "Unable to read file from '%s'. Failed with exception: %s",
+                file_path,
+                e,
+            )
+            raise
+        else:
+            logger.info("Unable to read file '%s' due to an exception: %s. Using default: %s", file_path, e, default)
+            return default
 
 
 def get_clustermgtd_heartbeat(clustermgtd_heartbeat_file_path):
